
/*** GLOBAL VARIABLES ***/
var _model = {};
var _currentState = null;
var _currentModelState = null;

var _map = null;
var _markers = [];
var _paths = [];
var _infowindow = null;

var _animatePositionShift = false;
var _animateDrop = false;
var _enableScrollDelay = false;

var _dropMarkerDelay = 100;
var _adjustZoomDelay = 400;

var _zoomOutValue = 4;
var _textOffset = 0.1;



/*** MODEL INITIALIZATION FUNCTION ***/
//Initialize model
/*** OK ***/
function initModel(){ //I-2
    //Replace with AJAX from azure Cloud service
    var response = states;
    $("#text-content").empty();
    $("#timeline-content").empty();

    //Create hashmap from response
    response.forEach(function(entry) {
        _model[key(entry)] = entry;

        if(entry.state == 1){
            //Append active text
            $("#text-content").append('<span id="position-' + entry.state + '" class="waypoint active">' + entry.text + '</span>');
            //Append active timeline
            $("#timeline-content").append('<li id="timeline-' + entry.state + '" class="active"><span class="glyphicon glyphicon-asterisk"></span> ' + entry.date + '</li>');       
        }else{
            //Append text
            $("#text-content").append('<span id="position-' + entry.state + '" class="waypoint">' + entry.text + '</span>');
            //Append timeline
            $("#timeline-content").append('<li id="timeline-' + entry.state + '"><span class="glyphicon glyphicon-asterisk"></span> ' + entry.date + '</li>');  
        }
    });

    initText();
}



/*** TEXT INITIALIZATION FUNCTIONS ***/
/*** OK ***/
//Initialize text
function initText(){ //I-3
    addTopPadding();
    displayText();
    addBottomPadding();
    addWaypoints();

    //Trigger first event
    $("#text-content").scrollTop(20);
}

//Add top padding to textview
/*** OK ***/
function addTopPadding(){ //I-4
    var textContainerHeight = $("#text-content").height();
    var topPaddingHeight = Math.floor(textContainerHeight*_textOffset);
    $("#text-content").prepend('<div style="height: ' + topPaddingHeight+ 'px;"></div>');   
}

//Add bottom padding to textview
/*** OK ***/
function addBottomPadding(){ //I-6
    var textContainerHeight = $("#text-content").height();
    var bottomPaddingHeight = Math.ceil(textContainerHeight - 
        textContainerHeight*_textOffset -  $("#text-content .waypoint").last().height() + 10);
    $("#text-content").append('<div style="height: ' + Math.max(bottomPaddingHeight,50) + 'px;"></div>');   
}

//Display text
/*** OK ***/
function displayText(){ //I-5
    var textContainerHeight = $("#text-content").height();
    $("#text-content").show();
    $("#caret").css("top", Math.floor(textContainerHeight*_textOffset) + "px");
    $("#caret").show();
}

//Add waypoint on spans
/*** OK ***/
function addWaypoints(){ //I-7
    var waypoints = document.getElementsByClassName('waypoint')
    for (var i = 0; i < waypoints.length; i++) {
        new Waypoint({
            element: waypoints[i],
            handler: function() {
              var id = this.element.id;
              $("#text-content > .waypoint").removeClass("active");
              $("#" + id).addClass("active");
              updateModel(id);
            },
            context: document.getElementById('text-content'),
            offset: (_textOffset*100).toString() + '%'
        })
    }
}



/*** MAP INITIALIZATION FUNCTIONS ***/
/*** OK ***/
//Initialize google Map(Startup)
function initMap() { //I-1
    _map = new google.maps.Map(document.getElementById('map'), {
        zoom: 5,
        center: {lat: 0, lng: 0},
        mapTypeId: 'terrain'
    });

    _infowindow = new google.maps.InfoWindow({
        content: "loading...",
        disableAutoPan: true
    });

    google.maps.event.addListenerOnce(_map, 'idle', function(){
        initModel();
    });
}



/*************************************************************/



/*** MODEL UPDATE FUNCTION **/
function updateModel (stateString) { //E-1 Called on waypoint event
    var state = (stateString.replace("position-", ""));

    //Return if current state is triggered
    if(state == _currentState) return;
    
    _currentState = state;
    _currentModelState = _model[_currentState];

    //Disable scrollview
    disableScrollngTextView();

    //Update view
    updateTitle();
    updateMap();
    updateTimeline();
}



/*** TEXT UPDATE FUNCTION **/
/*** TODO ***/
function updateTitle () {
    //TODO SET TITLE ON PAGE
    $("#title").html(_currentModelState.title);
}



/*** TIMELINE UPDATE FUNCTIONS ***/
/*** OK ***/
function updateTimeline () {
    var container = $("#timeline-container");
    var content = $("#timeline-content");

    content.children().removeClass("active");
    var timelineString = "#timeline-" + (_currentState);
    var activeTimelineElement = $(timelineString).addClass("active");

    //Calculate offset
    var containerWidth = container.width();
    var offset = activeTimelineElement.width() * (_currentState);

    //Adjust scroll offset
    if((offset)>=containerWidth){
        container.scrollLeft(offset-containerWidth);
    }
    else{
        container.scrollLeft(0);
    }
}


/*** MAP UPDATE FUNCTION ***/
/*** OK ***/
function updateMap () { //E-2
    //Shift position
    if(_animatePositionShift)
        shiftPositionAnimated(_currentModelState.zoom, _map.getZoom());
    else{
        _map.setZoom(_currentModelState.zoom);
        
        var center = new google.maps.LatLng(_currentModelState.centerPosition.lat, _currentModelState.centerPosition.lng);
        _map.setCenter(center);
    }

    //Reload map data
    deletePaths();
    reloadMarkers();
}



/*************************************************************/



/*** MODEL HELPER FUNCTION ***/
/*** OK ***/
//Accosiated array key
var key = function(obj){
    return obj.state;
};



/*** TEXT HELPER FUNCTION ***/
function disableScrollngTextView(){
    if(!_enableScrollDelay) return;
    $("#text-content").css("overflow-y", "hidden");
}

function enableScrollngTextView(timeout){
    if(!_enableScrollDelay) return;
    window.setTimeout(function() {
        $("#text-content").css("overflow-y", "scroll");
    }, timeout);
}

//Show image image if present
/*** OK ***/
function showImage (location) {
    if(!(typeof location.imageUrl === "undefined") && !(location.imageUrl == "")){
        $("#map").css("height", "60%");
        $("#locationImg").show();

        $("#locationImg").css("background-image", "url('" + location.imageUrl + "')");
        $("#locationImgTitle").html(location.title);

        if(!(typeof location.description === "undefined")){
            $("#locationImgDescription").html(location.description);  
        }
    }
    else{
        $("#map").css("height", "100%");
        $("#locationImg").hide();
    }
    
    //Refresh map
    google.maps.event.trigger(_map,'resize');
}


/*** MAP HELPER FUNCTION ***/
/*** OK ***/
//Reloads markers in map
function reloadMarkers() { //E-4
    deleteMarkers();
    for (var i = 0; i < _currentModelState.locations.length; i++) {
        addMarker(_currentModelState.locations[i]);
    }
}

//Deletes all markers in the array by removing references to them
/*** OK ***/
function deleteMarkers() { //E-5
    for (var i = 0; i < _markers.length; i++) {
        _markers[i].setMap(null);
    }
    _markers = [];
}

//Deletes all paths in the array by removing references to them
/*** OK ***/
function deletePaths() { //E-3
    for (var i = 0; i < _paths.length; i++) {
        _paths[i].setMap(null);
    }
    _paths = [];
}


//Add single marker with infowindow and trail
/*** OK ***/
function addMarker(location) { //E-6 //TODO timeout if needed
    var markerColor = "http://maps.google.com/mapfiles/ms/icons/red-dot.png";
    //Marker color
    if(!(typeof location.markerColor === "undefined") && !(location.markerColor == "")){
        markerColor = "http://maps.google.com/mapfiles/ms/icons/" + location.markerColor + "-dot.png";
    }
    console.log(markerColor);
    //Add marker
    var marker = new google.maps.Marker({
        position: {lat: location.position.lat, lng: location.position.lng},
        map: _map,
        animation: (_animateDrop) ? google.maps.Animation.DROP : null,
        html: location.title,
        icon : markerColor
    });

    _markers.push(marker);

    //Add infowindow content
    google.maps.event.addListener(marker, "click", function () {
        _infowindow.setContent(this.html);
        _infowindow.open(_map, this);

        showImage(location);

    });
    
    //Display infowindow and image if highligted
    if(location.isHighligted){
        _infowindow.setContent(marker.html);
        _infowindow.open(_map, marker);

        showImage(location);
    }

    //Draw path
    if(!(typeof location.path === "undefined") && location.path.length>0){
        var coordinates = [];
        for(var i = 0; i < location.path.length; i++){
            coordinates.push({lat: location.path[i].lat, lng: location.path[i].lng});
        }

        //Path color
        var pathColor = "#FF0000";
        if(!(typeof location.pathColor === "undefined") && !(location.pathColor == "")){
            pathColor = location.pathColor;
        }

        //Path thickness
        var pathThickness = "2";
        if(!(typeof location.pathThickness === "undefined") && !(location.pathThickness == "")){
            pathThickness = location.pathThickness;
        }

        var path = new google.maps.Polyline({
            path: coordinates,
            geodesic: true,
            strokeColor: pathColor,
            strokeOpacity: 1.0,
            strokeWeight: pathThickness
        });

        path.setMap(_map);
        _paths.push(path);
    }


    //Draw direction/action path
    if(!(typeof location.directionPath === "undefined")){
        var coordinates = [];
        coordinates.push({lat: location.position.lat, lng: location.position.lng});
        coordinates.push({lat: location.directionPath.lat, lng: location.directionPath.lng});
        
        var lineSymbol = {
            path: 'M 0,-1 0,1',
            strokeOpacity: 1,
            scale: 4
        };

        var path = new google.maps.Polyline({
            path: coordinates,
            strokeOpacity: 0,
            icons: [{
              icon: lineSymbol,
              offset: '0',
              repeat: '20px'
            }]
        });

        path.setMap(_map);
        _paths.push(path);
    }

    
}

//Animated zoom
function shiftPositionAnimated (target, current) {
    var zoomLevel = Math.min(target, _zoomOutValue);
    smoothZoomOut(zoomLevel, current);
}

//Animated zoom out
function smoothZoomOut (min, current, targetZoom) {
    if (current < min){
        setTimeout(function(){setCenterAnimated(targetZoom)}, _adjustZoomDelay*2);
        return;
    }
    else {
        z = google.maps.event.addListener(_map, 'zoom_changed', function(event){
            google.maps.event.removeListener(z);
            smoothZoomOut(_map, min, current - 1);
        });
        setTimeout(function(){_map.setZoom(current)}, _adjustZoomDelay);
    }
}

//Animated zoom in
function smoothZoomIn (map, max, current) {
    if (current > max) return;
    else {
        z = google.maps.event.addListener(map, 'zoom_changed', function(event){
            google.maps.event.removeListener(z);
            smoothZoomIn(map, max, current + 1);
        });
        setTimeout(function(){map.setZoom(current)}, _adjustZoomDelay);
    }
}

//Set new center position
function setCenterAnimated (targetZoom) {
    var center = new google.maps.LatLng(_currentModelState.centerPosition.lat, _currentModelState.centerPosition.lng);
    _map.panTo(center);
    
    //Zoom in on new center
    smoothZoomIn(_currentModelState.zoom, map.getZoom());
}



/*************************************************************/


/*** JSON RESPONSE MARKUP ***/
/*
var states = [
    {
        "state" : "1",
        "text" : "Kort tid etter ble det satt inn under Weserübung-Nord, det tyske overfallet på Norge. Den 6. april ankommer 800-900 soldater fra den 163. infantridivisjonen, med ammunisjon og utstyr. Skipet hadde enda ikke fått utført noen øvelser med hovedbatteriet, skadekontroll, eller kampstasjonsøvelser, og ble ikke ansett som fullt operativt. Skipet ble uansett satt inn grunnet mangelen på tyngre skip i Kriegsmarine. Den 7. april 1940 drar skipet fra Swinemünde med «Emden» og tre torpedobåter under kaptein Heinrich Woldag. Skipet møtte opp med «Lutzow» på veien. Det var førende skip i invasjonsgruppe 5 (under kontreadmiral Oskar Kummetz), og sammen med krysserne «Lützow» og «Emden», og noen mindre skip, brakte det invasjonsstyrker bestemt for Oslo.",
        "title" : "Blucher sinks",
        "date" : "07/04-1940 12:00",
        "centerPosition" : { "lat" : 53.8846446, "lng" : 14.241223 },
        "zoom" : 5,
        "locations" : [
            {
                "title" : "Blucher",
                "description" : "Leading crosser for occupation of Oslo, Norway",
                "position" : {"lat" : 59.699324686349016, "lng" : 10.591850280761717},
                "isVisible" : true,
                "isHighligted" : true,
                "path" : [
                    {"lat" : 53.92536826573739, "lng" : 14.279308319091797}, //Swinemunde
                    {"lat" : 54.73572186418134, "lng" : 13.590087890625}, //Adjustment
                    {"lat" : 54.39335222384586, "lng" : 11.49169921875}, //Adjustment
                    {"lat" : 54.677007155737826, "lng" : 10.90667724609375}, //Adjustment
                    {"lat" : 55.21335628339293, "lng" : 11.019287109375}, //Adjustment
                    {"lat" : 55.7765730186677,  "lng" : 10.76934814453125}, //Adjustment
                    {"lat" : 56.46249048388979, "lng" : 11.2445068359375}, //Adjustment
                    {"lat" : 57.33838126552897, "lng" : 11.337890625}, //Adjustment
                    {"lat" : 57.76279865944121, "lng" : 10.9259033203125}, //Attach by HMS triton(submarine)
                    {"lat" : 59.178742850970224, "lng" : 10.625152587890625}, //Fulehauk fyr
                    {"lat" : 59.22198818911461, "lng" : 10.6292724609375}, //Ouside Rauøy/Bolærne, Pol III alerts blucher
                    {"lat" : 59.43320477511503, "lng" : 10.564727783203125}, //Adjustment
                    {"lat" : 59.571809700912844, "lng" : 10.63270568847656}, //Ouside Filtvet fyr
                    {"lat" : 59.66598503603134, "lng" : 10.612878799438477}, //Hit from Oscarsborg, TURN
                    {"lat" : 59.676148587135856, "lng" : 10.611333847045897}, //Torpedo from Oscarsborg
                    {"lat" : 59.699324686349016, "lng" : 10.591850280761717} //Sinks at Askholmene
                ],
                "markerColor" : "red",
                "pathColor" : "#FF0000",
                "pathThickness" : "2"
            }
        ]
    },
    {
        "state" : "2",
        "text" : "På vei mot Oslo den 8. april blir Gruppe 5 angrepet av HMS «Triton» (N15)» fra Royal Navy. Alle britiske torpedoer blir unngått.",
        "title" : "Blücher is sunk on its way to oslo, 04:21 GMT+1 9. april 1940",
        "date" : "07/04-1940 04:00",
        "centerPosition" : { "lat" : 59.6680218, "lng" : 10.6134796 },
        "zoom" : 13,
        "locations" : [
            {
                "title" : "Oscarsborg festning",
                "description" : "Dock",
                "position" : { "lat" : 59.6734457, "lng" : 10.6071177 },
                "isVisible" : true,
                "isHighligted" : false,
                "imageUrl" : "img/1.jpg",
                "markerColor" : "green"
            },{
                "title" : "Torpedobatteri, Nordre Kaholmen",
                "description" : "Dock",
                "position" : { "lat" : 59.6784506, "lng" : 10.6086416 },
                "isVisible" : true,
                "isHighligted" : false,
                "markerColor" : "yellow"
            },{
                "title" : "Husvikbatteriet, Drøbak",
                "description" : "Dock",
                "position" : { "lat" : 59.6732962, "lng" : 10.6203854 },
                "isVisible" : true,
                "isHighligted" : false,
                "markerColor" : "purple"
            },{
                "title" : "Kopåsbatteriet, Drøbak",
                "description" : "Dock",
                "position" : { "lat" : 59.6719801, "lng" : 10.6233931 },
                "isVisible" : true,
                "isHighligted" : false,
                "imageUrl" : "img/4.jpg"
            },{
                "title" : "Blücher",
                "description" : "Dock",
                "position" : { "lat" : 59.6680218, "lng" : 10.6134796 },
                "isVisible" : true,
                "isHighligted" : false,
                "imageUrl" : "img/5.jpg",
                "path" : [
                    {"lat": 59.772, "lng": -10.214},
                    {"lat": 21.291, "lng": -157.821}
                ],
                "markerColor" : "green",
                "pathColor" : "#0000FF",
                "pathThickness" : "1"
            }
        ]
    }
];
*/

/*
    JSON RESPONSE MODEL
    {
        "state" : 0,
        "text" : "string",
        "title" : "string",
        "date" : "string",
        "centerPosition" : { "lat" : 0, "lng" : 0 },
        "zoom" : 0,
        "locations" : [
            {
                "title" : "string",
                "description" : "string",
                "position" : { "lat" : 0, "lng" : 0 },
                "isHighligted" : true,
                "imageUrl" : "string",
                "path" : [
                    {"lat": 0, "lng": 0}
                ],
                "markerColor" : "string",    [red, blue, green, purple, yellow]
                "pathColor" : "string",      [HEX]
                "pathThickness" : "string"
            }
        ]
    }

    C# RETURN MODEL

    public class Position
    {
        public int Lat { get; set; }
        public int Lng { get; set; }
    }

    public class Location
    {
        public string Title { get; set; }
        public string Description { get; set; }
        public Position Position { get; set; }
        public bool IsVisible { get; set; }
        public bool IsHighligted { get; set; }
        public string ImageUrl { get; set; }
        public List<Position> Path { get; set; }
        public string MarkerColor { get; set; }
        public string PathColor { get; set; }
        public string PathThickness { get; set; }
    }

    public class ReturnModel
    {
        public int State { get; set; }
        public string Text { get; set; }
        public string Title { get; set; }
        public string Date { get; set; }
        public Position CenterPosition { get; set; }
        public int Zoom { get; set; }
        public List<Location> Locations { get; set; }
    }
*/

